@@ -22,10 +22,15 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <malloc.h>
+#include <stdint.h>
+#include <limits.h>
 #include <sys/stat.h>
 #include <sys/file.h>
-#include <sys/mount.h>
+#include <sys/wait.h>
+#include <sys/user.h>
+#include <sys/ioctl.h>
 #include <linux/types.h>
+#include <linux/fs.h>
 #include <string.h>
 
 #include "ploop.h"
@@ -51,6 +56,8 @@ char *mntn2str(int mntn_type)
 		return "MERGE";
 	case PLOOP_MNTN_GROW:
 		return "GROW";
+	case PLOOP_MNTN_DISCARD:
+		return "DISCARD";
 	}
 
 	return "UNKNOWN";
@@ -88,6 +95,15 @@ static int ioctl_device(int fd, int req, void *arg)
 		case PLOOP_IOC_FBGET:
 			msg = "PLOOP_IOC_FBGET";
 			break;
+		case PLOOP_IOC_DISCARD_INIT:
+			msg = "PLOOP_IOC_DISCARD_INIT";
+			break;
+		case PLOOP_IOC_DISCARD_WAIT:
+			msg = "PLOOP_IOC_DISCARD_WAIT";
+			break;
+		case PLOOP_IOC_DISCARD_FINI:
+			msg = "PLOOP_IOC_DISCARD_FINI";
+			break;
 		}
 		ploop_err(errno, "%s", msg);
 		return(SYSEXIT_DEVIOC);
@@ -505,12 +521,11 @@ err:
 	return ret;
 }
 
-int ploop_baloon_complete(const char *device)
+static int ploop_balloon_relocation(int fd, struct ploop_balloon_ctl *b_ctl, const char *device)
 {
-	int    ret = -1, fd = -1;
+	int    ret = -1;
 	__u32  n_free_blocks = 0;
 	__u32  freezed_a_h;
-	struct ploop_balloon_ctl    b_ctl;
 	struct freemap		   *freemap = NULL;
 	struct freemap		   *rangemap = NULL;
 	struct relocmap		   *relocmap = NULL;
@@ -529,40 +544,19 @@ int ploop_baloon_complete(const char *device)
 		goto err;
 	}
 
-	fd = open_device(device);
-	if (fd == -1)
-		goto err;
+	top_level   = b_ctl->level;
+	freezed_a_h = b_ctl->alloc_head;
 
-	memset(&b_ctl, 0, sizeof(b_ctl));
-	b_ctl.keep_intact = 1;
-	ret = ioctl_device(fd, PLOOP_IOC_BALLOON, &b_ctl);
-	if (ret)
-		goto err;
+	if (b_ctl->mntn_type == PLOOP_MNTN_RELOC)
+		goto reloc;
 
-	switch (b_ctl.mntn_type) {
-	case PLOOP_MNTN_BALLOON:
-	case PLOOP_MNTN_MERGE:
-	case PLOOP_MNTN_GROW:
-	case PLOOP_MNTN_TRACK:
-	case PLOOP_MNTN_OFF:
-		ploop_log(0, "Nothing to complete: kernel is in \"%s\" state",
-			mntn2str(b_ctl.mntn_type));
-		return 0;
-	case PLOOP_MNTN_FBLOADED:
-	case PLOOP_MNTN_RELOC:
-		top_level   = b_ctl.level;
-		freezed_a_h = b_ctl.alloc_head;
-		break;
-	default:
-		ploop_err(0, "Error: unknown mntn_type (%u)",
-			b_ctl.mntn_type);
+	if (b_ctl->mntn_type != PLOOP_MNTN_FBLOADED) {
+		ploop_err(0, "Error: non-suitable mntn_type (%u)",
+			b_ctl->mntn_type);
 		ret = SYSEXIT_PROTOCOL;
 		goto err;
 	}
 
-	if (b_ctl.mntn_type == PLOOP_MNTN_RELOC)
-		goto reloc;
-
 	ret = freeblks_alloc(&freeblks, 0);
 	if (ret)
 		goto err;
@@ -614,7 +608,6 @@ reloc:
 			(unsigned long long)(relocblks->alloc_head * S2B(delta.blocksize)));
 err:
 
-	close(fd);
 	free(freemap);
 	free(rangemap);
 	free(relocmap);
@@ -625,7 +618,53 @@ err:
 	return ret;
 }
 
-int ploop_baloon_check_and_repair(const char *device, char *mount_point, int repair)
+int ploop_balloon_complete(const char *device)
+{
+	int fd, err;
+	struct ploop_balloon_ctl b_ctl;
+
+	fd = open_device(device);
+	if (fd == -1)
+		return -1;
+
+	err = ioctl_device(fd, PLOOP_IOC_DISCARD_FINI, NULL);
+	if (err && errno != EBUSY) {
+		ploop_err(errno, "Can't finalize discard mode");
+		goto out;
+	}
+
+	memset(&b_ctl, 0, sizeof(b_ctl));
+	b_ctl.keep_intact = 1;
+	err = ioctl_device(fd, PLOOP_IOC_BALLOON, &b_ctl);
+	if (err)
+		goto out;
+
+	switch (b_ctl.mntn_type) {
+	case PLOOP_MNTN_BALLOON:
+	case PLOOP_MNTN_MERGE:
+	case PLOOP_MNTN_GROW:
+	case PLOOP_MNTN_TRACK:
+	case PLOOP_MNTN_OFF:
+		ploop_log(0, "Nothing to complete: kernel is in \"%s\" state",
+			mntn2str(b_ctl.mntn_type));
+		goto out;
+	case PLOOP_MNTN_RELOC:
+	case PLOOP_MNTN_FBLOADED:
+		break;
+	default:
+		ploop_err(0, "Error: unknown mntn_type (%u)",
+			b_ctl.mntn_type);
+		err = SYSEXIT_PROTOCOL;
+		goto out;
+	}
+
+	err = ploop_balloon_relocation(fd, &b_ctl, device);
+out:
+	close(fd);
+	return err;
+}
+
+int ploop_balloon_check_and_repair(const char *device, char *mount_point, int repair)
 {
 	int   ret, fd = -1;
 	int   balloonfd = -1;
@@ -805,3 +844,116 @@ err:
 
 	return ret;
 }
+
+static int ploop_trim(const char *mount_point)
+{
+	struct fstrim_range range = {0, ULLONG_MAX, PAGE_SIZE};
+	int fd, ret;
+
+	fd = open(mount_point, O_RDONLY);
+	if (fd < 0) {
+		ploop_err(errno, "Can't open mount_point");
+		return -1;
+	}
+
+	ret = ioctl(fd, FITRIM, &range);
+	if (ret < 0)
+		ploop_err(errno, "Can't trim file system");
+
+	close(fd);
+
+	return ret;
+}
+
+int ploop_discard(const char *device, const char *mount_point)
+{
+	int fd, ret, status;
+	int exit_code = 0;
+	pid_t tpid;
+
+	fd = open_device(device);
+	if (fd == -1)
+		return SYSEXIT_OPEN;
+
+	ret = ioctl_device(fd, PLOOP_IOC_DISCARD_INIT, NULL);
+	if (ret) {
+		ploop_err(errno, "Can't initialize discard mode");
+		close(fd);
+		return 1;
+	}
+
+	tpid = fork();
+	if (tpid < 0) {
+		ploop_err(errno, "Can't fork");
+		if (ioctl_device(fd, PLOOP_IOC_DISCARD_FINI, NULL))
+			ploop_err(errno, "Can't finalize discard mode");
+
+		close(fd);
+		return -1;
+	}
+
+	if (tpid == 0) {
+		ret = ploop_trim(mount_point);
+		if (ret < 0)
+			exit_code = 1;
+
+		if (ioctl_device(fd, PLOOP_IOC_DISCARD_FINI, NULL))
+			ploop_err(errno, "Can't finalize discard mode");
+
+		close(fd);
+		exit(exit_code);
+	}
+
+	while (1) {
+		struct ploop_balloon_ctl b_ctl;
+
+		ploop_log(0, "Waiting");
+		ret = ioctl_device(fd, PLOOP_IOC_DISCARD_WAIT, NULL);
+		if (ret) {
+			ploop_err(errno, "Waiting for a discard request failed");
+			break;
+		}
+
+		memset(&b_ctl, 0, sizeof(b_ctl));
+		b_ctl.keep_intact = 1;
+		ret = ioctl_device(fd, PLOOP_IOC_BALLOON, &b_ctl);
+		if (ret)
+			break;
+
+		if (b_ctl.mntn_type == PLOOP_MNTN_OFF)
+			break;
+
+		ploop_log(0, "Start relocation");
+		ret = ploop_balloon_relocation(fd, &b_ctl, device);
+		if (ret)
+			break;
+	}
+
+	if (ret) {
+		exit_code = 1;
+
+		ret = ioctl_device(fd, PLOOP_IOC_DISCARD_FINI, NULL);
+		if (ret < 0)
+			ploop_err(errno, "Can't finalize discard mode");
+
+		kill(tpid, SIGKILL);
+	}
+
+	close(fd);
+
+	ret = waitpid(tpid, &status, 0);
+	if (ret == -1) {
+		ploop_err(errno, "wait() failed");
+		exit_code = 1;
+	} else if(!WIFEXITED(status) || WEXITSTATUS(status)) {
+		if (WIFEXITED(status))
+			ploop_err(0, "The trim process failed with code %d",
+							WEXITSTATUS(status));
+		else
+			ploop_err(0, "The trim process killed by signal %d",
+							WTERMSIG(status));
+		exit_code = 1;
+	}
+
+	return exit_code;
+}
