@@ -329,9 +329,16 @@ static int create_empty_preallocated_delta(const char *path, __u32 blocksize, of
 	memcpy(buf, &vh, sizeof(struct ploop_pvd_header));
 	vh.m_Flags = CIF_Empty;
 
-	if (sys_fallocate(odelta.fd, 0, 0, S2B(vh.m_FirstBlockOffset + vh.m_SizeInSectors))) {
-		ploop_err(errno, "Failed to fallocate");
-		goto out_close;
+	rc = sys_fallocate(odelta.fd, 0, 0, S2B(vh.m_FirstBlockOffset + vh.m_SizeInSectors));
+	if (rc) {
+		if (errno == ENOTSUP) {
+			ploop_log(0, "Warning: fallocate is not supported, using truncate instead");
+			rc = ftruncate(odelta.fd, S2B(vh.m_FirstBlockOffset + vh.m_SizeInSectors));
+		}
+		if (rc) {
+			ploop_err(errno, "Failed to create %s", path);
+			goto out_close;
+		}
 	}
 
 	for (clu = 0; clu < SizeToFill / cluster; clu++) {
@@ -877,6 +884,20 @@ int ploop_find_dev_by_uuid(struct ploop_disk_images_data *di,
 	return ret;
 }
 
+int ploop_get_dev(struct ploop_disk_images_data *di, char *out, int len)
+{
+	int ret;
+
+	if (ploop_lock_di(di))
+		return SYSEXIT_LOCK;
+
+	ret = ploop_find_dev(di->runtime->component_name, di->images[0]->file, out, len);
+
+	ploop_unlock_di(di);
+
+	return ret;
+}
+
 static int reread_part(char *device)
 {
 	int fd;
@@ -895,7 +916,9 @@ static int reread_part(char *device)
 
 static int ploop_mount_fs(struct ploop_mount_param *param)
 {
-	unsigned long flags = param->ro ? MS_RDONLY : 0;
+	unsigned long flags =
+		(param->flags & MS_NOATIME) |
+		(param->ro ? MS_RDONLY : 0);
 	char buf[MAXPATHLEN + sizeof(BALLOON_FNAME)];
 	struct stat st;
 	char *fstype = param->fstype == NULL ? DEFAULT_FSTYPE : param->fstype;
@@ -1100,7 +1123,6 @@ int ploop_mount(struct ploop_disk_images_data *di, char **images,
 	int i;
 	int ret = 0;
 	__u32 blocksize = 0;
-	__u32 prev_blocksize = 0;
 
 	if (images == NULL || images[0] == NULL) {
 		ploop_err(0, "ploop_mount: no deltas to mount");
@@ -1112,34 +1134,41 @@ int ploop_mount(struct ploop_disk_images_data *di, char **images,
 		return SYSEXIT_PARAM;
 	}
 
+	if (raw) {
+		if (param->blocksize)
+			blocksize = param->blocksize;
+		else if (di)
+			blocksize = di->blocksize;
+		else {
+			ploop_err(0, "Blocksize is not specified");
+			return SYSEXIT_PARAM;
+		}
+	} else if (di)
+		blocksize = di->blocksize;
+
 	for (i = 0; images[i] != NULL; i++) {
 		int ro;
+		int flags = FSCK_CHECK | (di ? FSCK_DROPINUSE : 0);
+		__u32 cur_blocksize;
 
 		if (raw && i == 0)
 			continue;
 
 		ro  = (images[i+1] != NULL || param->ro) ? 1 : 0;
-		ret = ploop_fsck(images[i], 0, 0, 1, ro, 0, &blocksize);
+		ret = ploop_fsck(images[i], flags, ro, 0, &cur_blocksize);
 		if (ret) {
 			ploop_err(0, "%s (%s): irrecoverable errors",
 					images[i], ro ? "ro" : "rw");
 			goto err;
 		}
-		if (raw && i == 1)
-			prev_blocksize = blocksize;
-		if (i != 0 && prev_blocksize != blocksize) {
-			ploop_err(0, "Incorrect blocksize %s bs=%d [prev bs=%d]",
-					images[i], blocksize, prev_blocksize);
+		if (blocksize == 0)
+			blocksize = cur_blocksize;
+		if (cur_blocksize != blocksize) {
+			ploop_err(0, "Incorrect blocksize %s bs=%d [current bs=%d]",
+					images[i], blocksize, cur_blocksize);
 			ret = SYSEXIT_PARAM;
 			goto err;
 		}
-		prev_blocksize = blocksize;
-	}
-	if (di != NULL && (raw && i != 1) && di->blocksize != blocksize) {
-		 ploop_err(0, "Inconsistent di blocksize=%d != image blocksize=%d",
-				 di->blocksize, blocksize);
-		 ret = SYSEXIT_PARAM;
-		 goto err;
 	}
 
 	ret = add_deltas(di, images, param, raw, blocksize, &lfd);
@@ -1386,6 +1415,7 @@ int ploop_resize_image(struct ploop_disk_images_data *di, struct ploop_resize_pa
 	off_t dev_size = 0;
 	__u64 balloon_size = 0;
 	__u64 new_balloon_size = 0;
+	struct statfs fs;
 
 	if (di->nimages == 0) {
 		ploop_err(0, "No images in DiskDescriptor");
@@ -1421,7 +1451,6 @@ int ploop_resize_image(struct ploop_disk_images_data *di, struct ploop_resize_pa
 	balloon_size = bytes2sec(st.st_size);
 
 	if (param->size == 0) {
-		struct statfs fs;
 		int delta = 1024 * 1024;
 
 		/* Inflate balloon up to max free space */
@@ -1455,10 +1484,26 @@ int ploop_resize_image(struct ploop_disk_images_data *di, struct ploop_resize_pa
 			goto err;
 		tune_fs(mount_param.target, mount_param.device, param->size);
 	} else {
+		off_t available_balloon_size;
 		// SHRINK
 		/* FIXME: resize file system in case fs_size != dev_size
 		 */
+		if (statfs(mount_param.target, &fs) != 0) {
+			ploop_err(errno, "statfs(%s)", mount_param.target);
+			ret = SYSEXIT_FSTAT;
+			goto err;
+		}
 		new_balloon_size = dev_size - param->size;
+		available_balloon_size = balloon_size + B2S(fs.f_bfree * fs.f_bsize);
+		if (available_balloon_size < new_balloon_size) {
+			ploop_err(0, "Unable to change image size to %llu "
+					"sectors, minimal size is %lu",
+					param->size,
+					(long unsigned)(dev_size - available_balloon_size));
+			ret = SYSEXIT_PARAM;
+			goto err;
+		}
+
 		if (new_balloon_size != balloon_size) {
 			ret = ploop_balloon_change_size(mount_param.device,
 					balloonfd, new_balloon_size);
@@ -1599,9 +1644,14 @@ static int expanded2preallocated(struct ploop_disk_images_data *di)
 
 			delta.l2[l2_slot] = data_off * delta.blocksize;
 
-			if (sys_fallocate(delta.fd, 0, data_off * cluster, cluster)) {
-				ploop_err(errno, "Failed to fallocate");
-				goto err;
+			ret = sys_fallocate(delta.fd, 0, data_off * cluster, cluster);
+			if (ret) {
+				if (errno == ENOTSUP)
+					ret = ftruncate(delta.fd, (data_off + 1) * cluster);
+				if (ret) {
+					ploop_err(errno, "Failed to expand %s", di->images[0]->file);
+					goto err;
+				}
 			}
 
 			if (PWRITE(&delta, &delta.l2[l2_slot], sizeof(__u32), idx_off))
@@ -1656,9 +1706,8 @@ int ploop_get_info(struct ploop_disk_images_data *di, struct ploop_info *info)
 			get_mount_dir(dev, mnt, sizeof(mnt)) == 0)
 	{
 		ploop_unlock_di(di);
-		if (get_statfs_info(mnt, info))
-			return -1;
-		return 0;
+		if (get_statfs_info(mnt, info) == 0)
+			return 0;
 	}
 
 	ploop_unlock_di(di);
@@ -1914,7 +1963,7 @@ int ploop_switch_snapshot(struct ploop_disk_images_data *di, const char *guid, i
 		goto err_cleanup3;
 	}
 	if (old_top_delta_fname != NULL) {
-		ploop_log(0, "delete %s", old_top_delta_fname);
+		ploop_log(0, "Removing %s", old_top_delta_fname);
 		if (unlink(old_top_delta_fname))
 			ploop_err(errno, "Can't unlink %s",
 					old_top_delta_fname);
