@@ -140,6 +140,14 @@ static int parse_xml(const char *basedir, xmlNode *root_node, struct ploop_disk_
 		if (parse_ul(data, &val) == 0)
 			di->sectors= (unsigned)val;
 	}
+	cur_node = seek(root_node, "/StorageData/Storage");
+	ERR(cur_node, "/StorageData/Storage");
+	node = seek(cur_node, "Blocksize");
+	if (node != NULL) {
+		data = get_element_txt(node);
+		if (parse_ul(data, &val) == 0)
+			di->blocksize = (unsigned)val;
+	}
 
 	cur_node = seek(root_node, "/StorageData/Storage/Image");
 	ERR(cur_node, "/StorageData/Storage/Image");
@@ -177,8 +185,8 @@ static int parse_xml(const char *basedir, xmlNode *root_node, struct ploop_disk_
 		if (ploop_add_image_entry(di, image, guid))
 			return -1;
 	}
-	cur_node = seek(root_node, "/SnapshotTree");
-	ERR(cur_node, "/SnapshotTree");
+	cur_node = seek(root_node, "/Snapshots");
+	ERR(cur_node, "/Snapshots");
 
 	node = seek(cur_node, "TopGUID");
 	if (node != NULL) {
@@ -187,7 +195,7 @@ static int parse_xml(const char *basedir, xmlNode *root_node, struct ploop_disk_
 		di->top_guid = strdup(data);
 	}
 
-	cur_node = seek(root_node, "/SnapshotTree/Shot");
+	cur_node = seek(root_node, "/Snapshots/Shot");
 	if (cur_node != NULL) {
 		for (; cur_node; cur_node = cur_node->next) {
 			if (cur_node->type != XML_ELEMENT_NODE)
@@ -232,6 +240,10 @@ static int validate_disk_descriptor(struct ploop_disk_images_data *di)
 				di->runtime->xml_fname);
 		return -1;
 	}
+	if (!is_valid_blocksize(di->blocksize)) {
+		ploop_err(0, "Invalid block size %d", di->blocksize);
+		return -1;
+	}
 	// FIXME: compatibility issue have to be removed before BETA
 	if (di->nimages != di->nsnapshots) {
 		int ret;
@@ -242,6 +254,14 @@ static int validate_disk_descriptor(struct ploop_disk_images_data *di)
 		if (di->top_guid == NULL)
 			di->top_guid = strdup(BASE_UUID);
 	}
+	if (di->top_guid == NULL && find_snapshot_by_guid(di, BASE_UUID) != -1) {
+		/* Parallels VM compatibility.
+		 * The top delta is hardcoded {5fbaabe3-6958-40ff-92a7-860e329aab41}
+		 */
+		ploop_err(0, "No TopGuid found, using " BASE_UUID);
+		di->top_guid = strdup(BASE_UUID);
+		di->runtime->vm_compat = 1;
+	}
 	if (!is_valid_guid(di->top_guid)) {
 		ploop_err(0, "Validation of %s failed: invalid top delta %s",
 				di->runtime->xml_fname, di->top_guid);
@@ -314,13 +334,13 @@ int ploop_store_diskdescriptor(const char *fname, struct ploop_disk_images_data
 	xmlTextWriterPtr writer = NULL;
 	xmlDocPtr doc = NULL;
 	char tmp[MAXPATHLEN];
-	char basedir[MAXPATHLEN];
+	char basedir[MAXPATHLEN] = "";
 
 	ploop_log(0, "Storing %s", fname);
 
 	get_basedir(fname, tmp, sizeof(tmp));
 
-	if (realpath(tmp, basedir) == NULL) {
+	if (*tmp && realpath(tmp, basedir) == NULL) {
 		ploop_err(errno, "Can't resolve %s", tmp);
 		return -1;
 	}
@@ -417,7 +437,8 @@ int ploop_store_diskdescriptor(const char *fname, struct ploop_disk_images_data
 		ploop_err(0, "Error at xmlTextWriter End");
 		goto err;
 	}
-	rc = xmlTextWriterWriteElement(writer, BAD_CAST "Blocksize", BAD_CAST "512");
+	rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST "Blocksize", "%d",
+			di->blocksize);
 	if (rc < 0) {
 		ploop_err(0, "Error at xmlTextWriter Blocksize");
 		goto err;
@@ -474,56 +495,13 @@ int ploop_store_diskdescriptor(const char *fname, struct ploop_disk_images_data
 		goto err;
 	}
 	/****************************************
-	  Compatibility entry. handle validation in PrlDiskDescriptor::OpenDescriptor()
-		<Shot>
-		 <GUID>{5fbaabe3-6958-40ff-92a7-860e329aab41}</GUID>
-		 <ParentGUID>{00000000-0000-0000-0000-000000000000}</ParentGUID>
-		</Shot>
+	 *	Snapshots
 	 ****************************************/
 	rc = xmlTextWriterStartElement(writer, BAD_CAST "Snapshots");
 	if (rc < 0) {
 		ploop_err(0, "Error at xmlTextWriter Snapshots");
 		goto err;
 	}
-	rc = xmlTextWriterStartElement(writer, BAD_CAST "Shot");
-	if (rc < 0) {
-		ploop_err(0, "Error at xmlTextWriter Shot");
-		goto err;
-	}
-
-	rc = xmlTextWriterWriteElement(writer, BAD_CAST "GUID",
-			BAD_CAST BASE_UUID);
-	if (rc < 0) {
-		ploop_err(0, "Error at xmlTextWriter GUID");
-		goto err;
-	}
-	rc = xmlTextWriterWriteElement(writer, BAD_CAST "ParentGUID",
-			BAD_CAST NONE_UUID);
-	if (rc < 0) {
-		ploop_err(0, "Error at xmlTextWriter ParentGUID");
-		goto err;
-	}
-	/*  Close Shot */
-	rc = xmlTextWriterEndElement(writer);
-	if (rc < 0) {
-		ploop_err(0, "Error at xmlTextWriterEndElement");
-		goto err;
-	}
-	/*  Close Snapshots */
-	rc = xmlTextWriterEndElement(writer);
-	if (rc < 0) {
-		ploop_err(0, "Error at xmlTextWriterEndElement");
-		goto err;
-	}
-
-	/****************************************
-	 *	Snapshot
-	 ****************************************/
-	rc = xmlTextWriterStartElement(writer, BAD_CAST "SnapshotTree");
-	if (rc < 0) {
-		ploop_err(0, "Error at xmlTextWriter SnapshotTree");
-		goto err;
-	}
 	if (di->top_guid != NULL) {
 		rc = xmlTextWriterWriteElement(writer, BAD_CAST "TopGUID",
 				BAD_CAST di->top_guid);
@@ -563,7 +541,7 @@ int ploop_store_diskdescriptor(const char *fname, struct ploop_disk_images_data
 			goto err;
 		}
 	}
-	/* Close SnapshotTree */
+	/* Close Snapshots */
 	rc = xmlTextWriterEndElement(writer);
 	if (rc < 0) {
 		ploop_err(0, "Error at xmlTextWriterEndElement");
