@@ -40,7 +40,7 @@ struct delta delta;
 
 void usage_summary(void)
 {
-	fprintf(stderr, "Usage: ploop-balloon { show | status | clear | change | complete | check | repair } ...\n"
+	fprintf(stderr, "Usage: ploop-balloon { show | status | clear | change | complete | check | repair | discard } ...\n"
 			"Use \"ploop-balloon cmd\" to get more info about cmd\n"
 		);
 }
@@ -92,7 +92,7 @@ static void usage_status(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon status -d DEVICE -m MOUNT_POINT\n"
 			"	ploop-balloon status -f -d DEVICE\n"
-			"	DEVICE	    := ploop device, f.e. /dev/ploop0\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
 			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"	-f	    - do not flock hidden balloon\n"
 			"Action: inquire current in-kernel status of maintenance\n"
@@ -142,7 +142,7 @@ static int pb_status(int argc, char **argv)
 static void usage_clear(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon clear -d DEVICE -m MOUNT_POINT\n"
-			"	DEVICE	    := ploop device, f.e. /dev/ploop0\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
 			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"Action: clear stale in-kernel \"BALLOON\" state of maintenance\n"
 		);
@@ -190,7 +190,7 @@ static void usage_change(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon change -s SIZE -d DEVICE -m MOUNT_POINT\n"
 			"	SIZE	    := NUMBER[kmg] (new size of balloon)\n"
-			"	DEVICE	    := ploop device, f.e. /dev/ploop0\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
 			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"Action: inflate or truncate hidden balloon (dependently on new_size vs. old_size)\n"
 		);
@@ -243,7 +243,7 @@ static int pb_change(int argc, char **argv)
 static void usage_complete(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon complete -d DEVICE -m MOUNT_POINT\n"
-			"	DEVICE	    := ploop device, f.e. /dev/ploop0\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
 			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"Action: complete previously interrupted balloon operation\n"
 			"	 (sensible only if kernel is in \"FBLOAD\" or \"RELOC\"\n"
@@ -281,13 +281,13 @@ static int pb_complete(int argc, char **argv)
 	if (ret)
 		return ret;
 
-	return ploop_baloon_complete(device);
+	return ploop_balloon_complete(device);
 }
 
 static void usage_check(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon check -d DEVICE -m MOUNT_POINT\n"
-			"	DEVICE	    := ploop device, f.e. /dev/ploop0\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
 			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"Action: check whether hidden balloon has free blocks\n"
 		);
@@ -296,7 +296,7 @@ static void usage_check(void)
 static void usage_repair(void)
 {
 	fprintf(stderr, "Usage: ploop-balloon repair -d DEVICE -m MOUNT_POINT\n"
-			"	DEVICE	    := ploop device, f.e. /dev/ploop0\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
 			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
 			"Action: repair hidden balloon (i.e. relocate all free blocks\n"
 			"	 present in hidden balloon (w/o inflate))\n"
@@ -339,10 +339,47 @@ static int pb_check_and_repair(int argc, char **argv, int repair)
 		return -1;
 	}
 
-	return ploop_baloon_check_and_repair(device, mount_point, repair);
+	return ploop_balloon_check_and_repair(device, mount_point, repair);
 
 }
 
+static void usage_discard(void)
+{
+	fprintf(stderr, "Usage: ploop-balloon discard -d DEVICE -m MOUNT_POINT\n"
+			"	DEVICE	    := ploop device, e.g. /dev/ploop0\n"
+			"	MOUNT_POINT := path where fs living on ploop device mounted to\n"
+			"Action: discard unused blocks from the image.\n"
+		);
+}
+
+static int pb_discard(int argc, char **argv)
+{
+	int i;
+
+	while ((i = getopt(argc, argv, "d:m:")) != EOF) {
+		switch (i) {
+		case 'd':
+			device = optarg;
+			break;
+		case 'm':
+			mount_point = optarg;
+			break;
+		default:
+			return -1;
+		}
+	}
+
+	argc -= optind;
+	argv += optind;
+
+	if (argc != 0 || device == NULL || mount_point == NULL) {
+		usage_discard();
+		return -1;
+	}
+
+	return ploop_discard(device, mount_point);
+}
+
 int main(int argc, char **argv)
 {
 	char *cmd;
@@ -372,6 +409,8 @@ int main(int argc, char **argv)
 		return pb_check_and_repair(argc, argv, 0); /* check only */
 	if (strcmp(cmd, "repair") == 0)
 		return pb_check_and_repair(argc, argv, 1); /* check and repair */
+	if (strcmp(cmd, "discard") == 0)
+		return pb_discard(argc, argv);
 
 	usage_summary();
 	return -1;
