@@ -35,11 +35,10 @@ static struct ploop_cancel_handle *_s_cancel_handle;
 void usage_summary(void)
 {
 	fprintf(stderr, "Usage: ploop init -s SIZE [-f FORMAT] NEW_DELTA\n"
-			"       ploop mount [-rP] [-f raw] -d DEVICE [ TOP_DELTA ... ] BASE_DELTA\n"
+			"       ploop mount [-rP] [-f raw] -d DEVICE BASE_DELTA [ ... TOP_DELTA ]\n"
 			"       ploop umount -d DEVICE\n"
 			"       ploop { delete | rm } -d DEVICE -l LEVEL\n"
 			"       ploop merge -d DEVICE [-l LEVEL[..TOP_LEVEL]]\n"
-			"       ploop shrink [-f raw] [-d DEVICE | [ TOP_DELTA... ] BASE_DELTA ]\n"
 			"       ploop fsck [-fcr] DELTA\n"
 			"       ploop getdev\n"
 			"       ploop resize -s SIZE BASE_DELTA\n"
@@ -143,15 +142,17 @@ static int plooptool_init(int argc, char **argv)
 
 static void usage_mount(void)
 {
-	fprintf(stderr, "Usage: ploop mount [-rP] [-f FORMAT] [-d DEVICE] [TOP_DELTA ... ] BASE_DELTA\n"
+	fprintf(stderr, "Usage: ploop mount [-rP] [-f FORMAT] [-b BLOCKSIZE] [-d DEVICE]\n"
+			"             BASE_DELTA [ ... TOP_DELTA ]\n"
 			"       ploop mount [-rP] [-m DIR] [-u UUID] DiskDescriptor.xml\n"
 			"       FORMAT := { raw | ploop1 }\n"
+			"       BLOCKSIZE := block size (for raw image format)\n"
 			"       DEVICE := ploop device, e.g. /dev/ploop0\n"
 			"       *DELTA := path to image file\n"
 			"       -r     - mount images read-only\n"
 			"       -P     - rescan partition table upon mount\n"
 			"       -t     - file system type\n"
-			"	-m     - target mount point\n"
+			"       -m     - target mount point\n"
 		);
 }
 
@@ -162,7 +163,7 @@ static int plooptool_mount(int argc, char **argv)
 	int base = 0;
 	struct ploop_mount_param mountopts = {};
 
-	while ((i = getopt(argc, argv, "rf:Pd:m:t:u:o:")) != EOF) {
+	while ((i = getopt(argc, argv, "rf:Pd:m:t:u:o:b:")) != EOF) {
 		switch (i) {
 		case 'd':
 			strncpy(mountopts.device, optarg, sizeof(mountopts.device)-1);
@@ -196,6 +197,16 @@ static int plooptool_mount(int argc, char **argv)
 		case 'o':
 			mountopts.mount_data = strdup(optarg);
 			break;
+		case 'b': {
+			  char * endptr;
+
+			  mountopts.blocksize = strtoul(optarg, &endptr, 0);
+			  if (optarg == endptr) {
+				  usage_mount();
+				  return -1;
+			  }
+			  break;
+		}
 		default:
 			usage_mount();
 			return -1;
@@ -507,9 +518,9 @@ static void usage_snapshot(void)
 {
 	fprintf(stderr, "Usage: ploop snapshot [-u <uuid>] DiskDescriptor.xml\n"
 			"       ploop snapshot [-F] -d DEVICE DELTA\n"
-			"         DEVICE := ploop device, e.g. /dev/ploop0\n"
-			"         DELTA := path to new image file\n"
-			"         -F    - synchronize file system before taking snapshot\n"
+			"       DEVICE := ploop device, e.g. /dev/ploop0\n"
+			"       DELTA := path to new image file\n"
+			"       -F     - synchronize file system before taking snapshot\n"
 		);
 }
 
@@ -571,7 +582,7 @@ static int plooptool_snapshot(int argc, char **argv)
 static void usage_snapshot_switch(void)
 {
 	fprintf(stderr, "Usage: ploop snapshot-switch -u <uuid> DiskDescriptor.xml\n"
-			"  -u <uuid>	snapshot uuid\n");
+			"       -u <uuid>     snapshot uuid\n");
 }
 
 static int plooptool_snapshot_switch(int argc, char **argv)
@@ -614,7 +625,7 @@ static int plooptool_snapshot_switch(int argc, char **argv)
 static void usage_snapshot_delete(void)
 {
 	fprintf(stderr, "Usage: ploop snapshot-delete -u <uuid> DiskDescriptor.xml\n"
-			"  -u <uuid>	snapshot uuid\n");
+			"       -u <uuid>     snapshot uuid\n");
 }
 
 static int plooptool_snapshot_delete(int argc, char **argv)
@@ -657,7 +668,7 @@ static int plooptool_snapshot_delete(int argc, char **argv)
 void usage_snapshot_merge(void)
 {
 	fprintf(stderr, "Usage: ploop snapshot-merge [-u <uuid>] DiskDescriptor.xml]\n"
-			"  -u <uuid>	snapshot uuid (top delta if not specified)\n");
+			"       -u <uuid>     snapshot uuid (top delta if not specified)\n");
 }
 
 int plooptool_snapshot_merge(int argc, char ** argv)
@@ -835,7 +846,7 @@ static void usage_info(void)
 
 static void print_info(struct ploop_info *info)
 {
-	printf("%11s %14s  %14s\n",
+	printf("%11s %14s %14s\n",
 			"resource", "Size", "Used");
 	printf("%11s %14llu %14llu\n",
 			"1k-blocks",
